/*
 * AtletiGUIApp.java
 */

package control;
import java.io.File;
import atletigui.AtletiConsoleView;
import atletigui.AtletiGUIView;
import model.*;

import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;

/**
 * The main class of the application.
 */
public class AtletiGUIApp extends SingleFrameApplication {
    //constants
    public static final boolean DEBUG = true;
    // elementi di vista
    private AtletiGUIView vista;
    private AtletiConsoleView console;
    // elementi di model
    private SquadraDiAtleti atleti;
    private Atleta atletaInUso;
    private File fileInUso;
    

    /**
     * At startup create and show the main frame of the application.
     */
    @Override protected void startup() {
        vista=new AtletiGUIView(this);
        show(vista);
        myStartup();
    }

    /**
     * This method is to initialize the specified window by injecting resources.
     * Windows shown in our application come fully initialized from the GUI
     * builder, so this additional configuration is not needed.
     */
    @Override protected void configureWindow(java.awt.Window root) {
    }

    /**
     * A convenient static getter for the application instance.
     * @return the instance of AtletiGUIApp
     */
    public static AtletiGUIApp getApplication() {
        return Application.getInstance(AtletiGUIApp.class);
    }

    /**
     * Main method launching the application.
     */
    public static void main(String[] args) {
        launch(AtletiGUIApp.class, args);
    }
    
    
    /**
     * Ricerca dell'atleta in base al pettorale
     * viene invocato dall'handler dell'evento click sul bottone cerca
     */
    public void ricercaPettorale(){
      try {
        int i = vista.getPettorale();
        vista.scriviStato("");
        try {
            atletaInUso=atleti.ricercaPettorale(i);
          vista.scriviRisultato(atletaInUso);
        }
        catch (Exception e) {
          vista.scriviStato("Non esiste un atleta con questo numero di pettorale");
        }
      }
      catch(Exception e) {
        vista.scriviStato("Il dato di pettorale deve essere un numero intero");
      }
    }
    
    /**
     * Carica i dati da file indicato con il filechhoser
     * invocato dall'handler dell'evento click sulla voce di menu
     * gestisce l'eccezione 
     */
    public void leggiDaFile(){
      fileInUso = vista.getFile();
      if (fileInUso!=null){
//      AtletiFiller.popolaAtletiViaFile(f);
        try{
          atleti=AtletiFiller.popolaAtletiViaFile(fileInUso,console);
        }
        catch(Exception e){
          vista.scriviStato(e.getMessage());
          console.scrivi("Eccezione su file");
          atleti=new SquadraDiAtleti(0);
        }
        console.scrivi(atleti.toString());
      }
    }
    
    public void salvaDati(){
        AtletiFileScrittore scrittore;
        atletaInUso.setPettorale(vista.getPettoraleAtleta());
        atletaInUso.setCognomeNome(vista.getCognomeNomeAtleta());
        atletaInUso.setPeso(vista.getPesoAtleta());
        scrittore=new AtletiFileScrittore(fileInUso);
        //scrivi su file tutti gli atleti della squadra
        for (int i=0;i<atleti.getNumeroAtleti();i++){
            scrittore.setPettorale(atleti.getAtleta(i).getPettorale());
            scrittore.setCognomeNome(atleti.getAtleta(i).getCognomeNome());
            scrittore.setPeso(atleti.getAtleta(i).getPeso());
        }
        scrittore.chiudi();
    }
    
    /**
     * istanzia gli oggetti necessari all'applicazione
     */
    public void myStartup(){
      console=new AtletiConsoleView(DEBUG);
      vista.nascondiPannelloAtleta();
      atleti=AtletiFiller.popolaAtleti();
    }
}
